module Ecoportal
  module API
    class Internal
      # @attr_reader client [Common::Client] a `Common::Client` object that holds the configuration of the api connection.
      class LoginProviders
        include Enumerable

        attr_reader :client

        # @param client [Common::Client] a `Common::Client` object that holds the configuration of the api connection.
        # @return [LoginProviders] an instance object ready to make login providers api requests.
        def initialize(client)
          @client = client
        end

        # Gets all the login providers via api request.
        # @return [Enumerable<LoginProvider>] an `Enumerable` with all login providers already wrapped as `LoginProvider` objects.
        def get_all
          response = @client.get("/login_providers")
          Common::WrappedResponse.new(response, Internal::LoginProvider)
        end

        def each(&block)
          return to_enum(:each) unless block
          get_all.each(&block)
        end

      end
    end
  end
end

require 'ecoportal/api/internal/login_provider'
